/*
Copyright (C) 1996-2001 Id Software, Inc.
Copyright (C) 2002-2009 John Fitzgibbons and others
Copyright (C) 2010-2014 QuakeSpasm developers

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef spingle__wad_h
#define spingle__wad_h

//===============
//   TYPES
//===============

#define CMP_NONE                0
#define CMP_LZSS                1

#define TYP_NONE                0
#define TYP_LABEL               1

#define TYP_LUMPY               64                              // 64 + grab command number
#define TYP_PALETTE             64
#define TYP_QTEX                65
#define TYP_QPIC                66
#define TYP_SOUND               67
#define TYP_MIPTEX              68

#define WADFILENAME "gfx.wad" //johnfitz -- filename is now hard-coded for honesty

typedef struct
{
	int32_t                 width, height;
	byte            data[4];                        // variably sized
} qpic_t;

typedef struct
{
	char            identification[4];              // should be WAD2 or 2DAW
	int32_t                 numlumps;
	int32_t                 infotableofs;
} wadinfo_t;

typedef struct
{
	int32_t                 filepos;
	int32_t                 disksize;
	int32_t                 size;                                   // uncompressed
	int8_t          type;
	int8_t          compression;
	uint8_t         pad1, pad2;
	char            name[16];                               // must be null terminated
} lumpinfo_t;

extern  int32_t                 wad_numlumps;
extern  lumpinfo_t      *wad_lumps;
extern  byte            *wad_base;

void    W_LoadWadFile(void);  //johnfitz -- filename is now hard-coded for honesty
void    W_CleanupName(const char *in, char *out);
lumpinfo_t      *W_GetLumpinfo(const char *name);
void    *W_GetLumpName(const char *name);
void    *W_GetLumpNum(int32_t num);

void SwapPic(qpic_t *pic);

#endif
